from .user import *
from .bot import *
from .extra import *
from .textchannel import *
from .dmchannel import *
from .message import *
from .server import *
from .embed import *
from .button import *
from .member import *
from .get import *
from .colour import *

Color = Colour